@@ -36,9 +36,7 @@ struct connection {
 	chunkqueue *write_queue;      /* a large queue for low-level write ( HTTP response ) [ file, mem ] */
 	chunkqueue *read_queue;       /* a small queue for low-level read ( HTTP request ) [ mem ] */
 
-	off_t bytes_written;          /* used by mod_accesslog, mod_rrd */
-	off_t bytes_written_cur_second; /* used by mod_accesslog, mod_rrd */
-	off_t bytes_read;             /* used by mod_accesslog, mod_rrd */
+	off_t bytes_written_cur_second; /* used by rate-limiting and mod_status */
 
 	int (* network_write)(struct connection *con, chunkqueue *cq, off_t max_bytes);
 	int (* network_read)(struct connection *con, chunkqueue *cq, off_t max_bytes);
@@ -138,8 +136,6 @@ typedef struct server_socket {
 
 typedef struct {
 	server_socket **ptr;
-
-	uint32_t size;
 	uint32_t used;
 } server_socket_array;
 
@@ -155,12 +151,6 @@ struct server {
 	/* buffers */
 	buffer *tmp_buf;
 
-	/* counters */
-	int con_opened;
-	int con_read;
-	int con_written;
-	int con_closed;
-
 	int max_fds;    /* max possible fds */
 	int max_fds_lowat;/* low  watermark */
 	int max_fds_hiwat;/* high watermark */
@@ -178,12 +168,14 @@ struct server {
 
 	/* members used at start-up or rarely used */
 
+	handler_t (* plugins_request_reset)(request_st *r);/*(for cgi.local-redir)*/
+
 	server_config srvconf;
 	void *config_data_base;
 
 	server_socket_array srv_sockets;
 	server_socket_array srv_sockets_inherited;
-	struct { void *ptr; uint32_t used; uint32_t size; } plugins;
+	struct { void *ptr; uint32_t used; } plugins;
 
 	unix_time64_t startup_ts;
 	unix_time64_t graceful_expire_ts;
@@ -193,6 +185,7 @@ struct server {
 	pid_t pid;
 	int stdin_fd;
 
+	const buffer *default_server_tag;
 	char **argv;
   #ifdef HAVE_PCRE2_H
 	void *match_data; /*(shared and reused)*/
