@@ -1,8 +1,6 @@
 #include "first.h"
 
 #include "fdevent.h"
-#include "buffer.h"
-#include "log.h"
 
 #include <sys/types.h>
 #include "sys-socket.h"
@@ -20,6 +18,9 @@
 #include <winsock2.h>
 #endif
 
+#include "ck.h"
+#define force_assert(x) ck_assert(x)
+
 #ifdef SOCK_CLOEXEC
 static int use_sock_cloexec;
 #endif
@@ -279,6 +280,20 @@ int fdevent_mkostemp(char *path, int flags) {
 }
 
 
+/* accept4() added in Linux x86 in kernel 2.6.28, but not in arm until 2.6.36
+ * https://lwn.net/Articles/789961/ */
+#if defined(__linux__) \
+ && (defined(__arm__) || defined(__thumb__) || defined(__arm64__))
+#ifdef __has_include
+#if __has_include(<sys/syscall.h>)
+#include <sys/syscall.h>
+#endif
+#endif
+#ifndef SYS_accept4
+#define accept4(a,b,c,d) ((errno = ENOTSUP), -1)
+#endif
+#endif
+
 int fdevent_accept_listenfd(int listenfd, struct sockaddr *addr, size_t *addrlen) {
 	int fd;
 	socklen_t len = (socklen_t) *addrlen;
@@ -433,6 +448,7 @@ pid_t fdevent_fork_execve(const char *name, char *argv[], char *envp[], int fdin
     signal(SIGTSTP, SIG_DFL);
   #endif
     signal(SIGPIPE, SIG_DFL);
+    signal(SIGUSR1, SIG_DFL);
 
     execve(name, argv, envp ? envp : environ);
 
@@ -530,11 +546,18 @@ int fdevent_connect_status(int fd) {
 
 
 #include <netinet/tcp.h>
-#if (defined(__APPLE__) && defined(__MACH__)) \
-  || defined(__FreeBSD__) || defined(__NetBSD__) \
+#if  defined(__FreeBSD__) || defined(__NetBSD__) \
   || defined(__OpenBSD__) || defined(__DragonFly__)
 #include <netinet/tcp_fsm.h>
 #endif
+#if defined(__APPLE__) && defined(__MACH__)
+#include <TargetConditionals.h> /* TARGET_OS_IPHONE, TARGET_OS_MAC */
+#if TARGET_OS_IPHONE            /* iOS, tvOS, or watchOS device */
+/*#define TCPS_CLOSE_WAIT 5*/   /* ??? which header contains this, if any ??? */
+#elif TARGET_OS_MAC             /* MacOS */
+#include <netinet/tcp_fsm.h>
+#endif
+#endif
 
 /* fd must be TCP socket (AF_INET, AF_INET6), end-of-stream recv() 0 bytes */
 int fdevent_is_tcp_half_closed(int fd) {
@@ -544,7 +567,7 @@ int fdevent_is_tcp_half_closed(int fd) {
     return (0 == getsockopt(fd, IPPROTO_TCP, TCP_CONNECTION_INFO, &tcpi, &tlen)
             && tcpi.tcpi_state == TCPS_CLOSE_WAIT);
   #elif defined(TCP_INFO) && defined(TCPS_CLOSE_WAIT)
-    /* FreeBSD, NetBSD (not present in OpenBSD or DragonFlyBSD) */
+    /* FreeBSD, NetBSD, OpenBSD (not present in DragonFlyBSD) */
     struct tcp_info tcpi;
     socklen_t tlen = sizeof(tcpi);
     return (0 == getsockopt(fd, IPPROTO_TCP, TCP_INFO, &tcpi, &tlen)
@@ -578,6 +601,7 @@ int fdevent_set_so_reuseaddr (const int fd, const int opt)
 
 #include <sys/stat.h>
 #include "ck.h"
+#include "log.h"
 __attribute_cold__ /*(convenience routine for use at config at startup)*/
 char *
 fdevent_load_file (const char * const fn, off_t *lim, log_error_st *errh, void *(malloc_fn)(size_t), void(free_fn)(void *))
