@@ -36,6 +36,8 @@ struct pollfd;          /* declaration */
 #if defined HAVE_PORT_H && defined HAVE_PORT_CREATE && defined(__sun)
 # define FDEVENT_USE_SOLARIS_PORT
 # include <port.h>
+/* Illumos epoll not supported by lighttpd; POLLRDHUP != EPOLLRDHUP on Illumos*/
+# undef FDEVENT_USE_LINUX_EPOLL
 #endif
 
 #if defined HAVE_SYS_EVENT_H && defined HAVE_KQUEUE
@@ -43,11 +45,6 @@ struct pollfd;          /* declaration */
 struct kevent;          /* declaration */
 #endif
 
-#if defined HAVE_LIBEV
-# define FDEVENT_USE_LIBEV
-struct ev_loop;         /* declaration */
-#endif
-
 #include "base_decls.h"
 #include "fdevent.h"    /* (*fdevent_handler) */
 
@@ -58,8 +55,7 @@ typedef enum {
     FDEVENT_HANDLER_LINUX_SYSEPOLL,
     FDEVENT_HANDLER_SOLARIS_DEVPOLL,
     FDEVENT_HANDLER_SOLARIS_PORT,
-    FDEVENT_HANDLER_FREEBSD_KQUEUE,
-    FDEVENT_HANDLER_LIBEV
+    FDEVENT_HANDLER_FREEBSD_KQUEUE
 } fdevent_handler_t;
 
 /**
@@ -103,9 +99,6 @@ struct fdevents {
     int kq_fd;
     struct kevent *kq_results;
   #endif
-  #ifdef FDEVENT_USE_LIBEV
-    struct ev_loop *libev_loop;
-  #endif
   #ifdef FDEVENT_USE_POLL
     struct pollfd *pollfds;
 
