@@ -19,6 +19,10 @@
 #endif
 #endif
 
+#if defined(__sun)
+#define __EXTENSIONS__
+#endif
+
 #ifndef __STDC_WANT_LIB_EXT1__
 #define __STDC_WANT_LIB_EXT1__ 1
 #endif
