@@ -179,13 +179,15 @@ log_buffer_vsprintf (buffer * const restrict b,
     size_t blen = buffer_clen(b);
     size_t bsp  = buffer_string_space(b)+1;
     char *s = b->ptr + blen;
-    size_t n;
+    unsigned int n;
 
     va_list aptry;
     va_copy(aptry, ap);
-    n = (size_t)vsnprintf(s, bsp, fmt, aptry);
+    n = (unsigned int)vsnprintf(s, bsp, fmt, aptry);
     va_end(aptry);
 
+    if ((int)n <= 0)
+        return;
     if (n < bsp)
         buffer_truncate(b, blen+n); /*buffer_commit(b, n);*/
     else {
@@ -193,15 +195,16 @@ log_buffer_vsprintf (buffer * const restrict b,
         vsnprintf(s, n+1, fmt, ap);
     }
 
-    size_t i;
+    unsigned int i;
     for (i = 0; i < n && ' ' <= s[i] && s[i] <= '~'; ++i) ;/*(ASCII isprint())*/
     if (i == n) return; /* common case; nothing to encode */
 
     /* need to encode log line
      * copy original line fragment, append encoded line to buffer, free copy */
-    char * const src = (char *)malloc(n);
-    memcpy(src, s, n); /*(note: not '\0'-terminated)*/
-    buffer_truncate(b, blen);
+    n -= i;
+    char * const src = (char *)ck_malloc(n);
+    memcpy(src, s+i, n); /*(note: not '\0'-terminated)*/
+    buffer_truncate(b, blen+i);
     buffer_append_string_c_escaped(b, src, n);
     free(src);
 }
@@ -229,7 +232,7 @@ static void
 log_error_write (const log_error_st * const errh, buffer * const restrict b)
 {
     if (errh->mode != FDLOG_SYSLOG) { /* FDLOG_FD FDLOG_FILE FDLOG_PIPE */
-        buffer_append_string_len(b, CONST_STR_LEN("\n"));
+        buffer_append_char(b, '\n');
         write_all(errh->fd, BUF_PTR_LEN(b));
     }
     else {
