@@ -31,6 +31,7 @@
 #include "mod_auth_api.h"
 #include "base.h"
 #include "base64.h"
+#include "fdevent.h"
 #include "http_header.h"
 #include "log.h"
 #include "plugin.h"
@@ -55,7 +56,7 @@ INIT_FUNC(mod_authn_gssapi_init) {
       { "gssapi", mod_authn_gssapi_check, NULL };
     static http_auth_backend_t http_auth_backend_gssapi =
       { "gssapi", mod_authn_gssapi_basic, NULL, NULL };
-    plugin_data *p = calloc(1, sizeof(*p));
+    plugin_data *p = ck_calloc(1, sizeof(*p));
 
     /* register http_auth_scheme_gssapi and http_auth_backend_gssapi */
     http_auth_scheme_gssapi.p_d = p;
@@ -160,12 +161,13 @@ static handler_t mod_authn_gssapi_send_400_bad_request (request_st * const r)
 __attribute_cold__
 static void mod_authn_gssapi_log_gss_error(log_error_st *errh, const char *file, unsigned int line, const char *func, const char *extra, OM_uint32 err_maj, OM_uint32 err_min)
 {
-    buffer * const msg = buffer_init_string(func);
+    buffer * const msg = buffer_init();
     OM_uint32 maj_stat, min_stat;
     OM_uint32 msg_ctx = 0;
     gss_buffer_desc status_string;
 
-    buffer_append_string_len(msg, CONST_STR_LEN("("));
+    buffer_copy_string(msg, func);
+    buffer_append_char(msg, '(');
     if (extra) buffer_append_string(msg, extra);
     buffer_append_string_len(msg, CONST_STR_LEN("):"));
 
@@ -183,7 +185,7 @@ static void mod_authn_gssapi_log_gss_error(log_error_st *errh, const char *file,
         if (!GSS_ERROR(maj_stat)) {
             buffer_append_string_len(msg, CONST_STR_LEN(" ("));
             buffer_append_string(msg, status_string.value);
-            buffer_append_string_len(msg, CONST_STR_LEN(")"));
+            buffer_append_char(msg, ')');
             gss_release_buffer(&min_stat, &status_string);
         }
     } while (!GSS_ERROR(maj_stat) && msg_ctx != 0);
@@ -203,19 +205,12 @@ static void mod_authn_gssapi_log_krb5_error(log_error_st *errh, const char *file
 
 static int mod_authn_gssapi_create_krb5_ccache(request_st * const r, plugin_data * const p, krb5_context kcontext, krb5_principal princ, krb5_ccache * const ccache)
 {
-    buffer * const kccname = buffer_init_string("FILE:/tmp/krb5cc_gssapi_XXXXXX");
-    char * const ccname    = kccname->ptr + sizeof("FILE:")-1;
-    const size_t ccnamelen = buffer_clen(kccname)-(sizeof("FILE:")-1);
+    char kccname[]         = "FILE:/tmp/krb5cc_gssapi_XXXXXX";
+    char * const ccname    = kccname + sizeof("FILE:")-1;
     /*(future: might consider using server.upload-dirs instead of /tmp)*/
-  #ifdef __COVERITY__
-    /* POSIX-2008 requires mkstemp create file with 0600 perms */
-    umask(0600);
-  #endif
-    /* coverity[secure_temp : FALSE] */
-    int fd = mkstemp(ccname);
+    int fd = fdevent_mkostemp(ccname, 0);
     if (fd < 0) {
         log_perror(r->conf.errh, __FILE__, __LINE__, "mkstemp(): %s", ccname);
-        buffer_free(kccname);
         return -1;
     }
     close(fd);
@@ -223,7 +218,7 @@ static int mod_authn_gssapi_create_krb5_ccache(request_st * const r, plugin_data
     do {
         krb5_error_code problem;
 
-        problem = krb5_cc_resolve(kcontext, kccname->ptr, ccache);
+        problem = krb5_cc_resolve(kcontext, kccname, ccache);
         if (problem) {
             mod_authn_gssapi_log_krb5_error(r->conf.errh, __FILE__, __LINE__, "krb5_cc_resolve", NULL, kcontext, problem);
             break;
@@ -231,13 +226,14 @@ static int mod_authn_gssapi_create_krb5_ccache(request_st * const r, plugin_data
 
         problem = krb5_cc_initialize(kcontext, *ccache, princ);
         if (problem) {
-            mod_authn_gssapi_log_krb5_error(r->conf.errh, __FILE__, __LINE__, "krb5_cc_initialize", kccname->ptr, kcontext, problem);
+            mod_authn_gssapi_log_krb5_error(r->conf.errh, __FILE__, __LINE__, "krb5_cc_initialize", kccname, kcontext, problem);
             break;
         }
 
-        r->plugin_ctx[p->id] = kccname;
-
-        http_header_env_set(r, CONST_STR_LEN("KRB5CCNAME"), ccname, ccnamelen);
+        buffer * const vb = http_header_env_set_ptr(r, CONST_STR_LEN("KRB5CCNAME"));
+        r->plugin_ctx[p->id] = vb;
+        const size_t ccnamelen = sizeof(kccname)-sizeof("FILE:");
+        buffer_copy_string_len(vb, ccname, ccnamelen);
         http_header_request_set(r, HTTP_HEADER_OTHER, CONST_STR_LEN("X-Forwarded-Keytab"), ccname, ccnamelen);
 
         return 0;
@@ -249,7 +245,6 @@ static int mod_authn_gssapi_create_krb5_ccache(request_st * const r, plugin_data
         *ccache = NULL;
     }
     unlink(ccname);
-    buffer_free(kccname);
 
     return -1;
 }
@@ -416,17 +411,14 @@ static handler_t mod_authn_gssapi_check_spnego(request_st * const r, plugin_data
         goto end;
     }
 
-    if (!(acc_flags & GSS_C_CONF_FLAG)) {
-        log_error(r->conf.errh, __FILE__, __LINE__, "No confidentiality for user: %s", (char *)token_out.value);
-        goto end;
-    }
-
     /* check the allow-rules */
     if (!http_auth_match_rules(require, token_out.value, NULL, NULL)) {
         goto end;
     }
 
     if (p->conf.auth_gssapi_store_creds) {
+        if (!(acc_flags & GSS_C_CONF_FLAG))
+            log_error(r->conf.errh, __FILE__, __LINE__, "No confidentiality for user: %s", (char *)token_out.value);
         if (!(acc_flags & GSS_C_DELEG_FLAG)) {
             log_error(r->conf.errh, __FILE__, __LINE__, "Unable to delegate credentials for user: %s", (char *)token_out.value);
             goto end;
@@ -693,7 +685,7 @@ static handler_t mod_authn_gssapi_basic(request_st * const r, void *p_d, const h
     }
 
     if (strchr(username->ptr, '@') == NULL) {
-        user_at_realm = buffer_init_buffer(username);
+        buffer_copy_buffer((user_at_realm = buffer_init()), username);
         buffer_append_str2(user_at_realm, CONST_STR_LEN("@"),
                                           BUF_PTR_LEN(require->realm));
     }
@@ -793,16 +785,17 @@ static handler_t mod_authn_gssapi_basic(request_st * const r, void *p_d, const h
 
 REQUEST_FUNC(mod_authn_gssapi_handle_reset) {
     plugin_data *p = (plugin_data *)p_d;
-    buffer *kccname = (buffer *)r->plugin_ctx[p->id];
-    if (NULL != kccname) {
+    buffer * const ccname = (buffer *)r->plugin_ctx[p->id];
+    if (NULL != ccname) {
         r->plugin_ctx[p->id] = NULL;
-        unlink(kccname->ptr+sizeof("FILE:")-1);
-        buffer_free(kccname);
+        unlink(ccname->ptr);
     }
 
     return HANDLER_GO_ON;
 }
 
+
+__attribute_cold__
 int mod_authn_gssapi_plugin_init(plugin *p);
 int mod_authn_gssapi_plugin_init(plugin *p) {
     p->version     = LIGHTTPD_VERSION_ID;
