@@ -56,7 +56,7 @@ static handler_t mod_authn_ldap_basic(request_st * const r, void *p_d, const htt
 INIT_FUNC(mod_authn_ldap_init) {
     static http_auth_backend_t http_auth_backend_ldap =
       { "ldap", mod_authn_ldap_basic, NULL, NULL };
-    plugin_data *p = calloc(1, sizeof(*p));
+    plugin_data *p = ck_calloc(1, sizeof(*p));
 
     /* register http_auth_backend_ldap */
     http_auth_backend_ldap.p_d = p;
@@ -238,8 +238,7 @@ SETDEFAULTS_FUNC(mod_authn_ldap_set_defaults) {
                     buffer *b;
                     *(const buffer **)&b = cpv->v.b;
                     mod_authn_add_scheme(srv, b);
-                    ldc = calloc(1, sizeof(plugin_config_ldap));
-                    force_assert(ldc);
+                    ldc = ck_calloc(1, sizeof(plugin_config_ldap));
                     ldc->errh = srv->errh;
                     ldc->auth_ldap_hostname = b->ptr;
                     cpv->v.v = ldc;
@@ -815,6 +814,8 @@ static handler_t mod_authn_ldap_basic(request_st * const r, void *p_d, const htt
     return rc;
 }
 
+
+__attribute_cold__
 int mod_authn_ldap_plugin_init(plugin *p);
 int mod_authn_ldap_plugin_init(plugin *p) {
     p->version     = LIGHTTPD_VERSION_ID;
