@@ -99,14 +99,10 @@ typedef struct cgi_pid_t {
 } cgi_pid_t;
 
 static handler_ctx * cgi_handler_ctx_init(void) {
-	handler_ctx *hctx = calloc(1, sizeof(*hctx));
-
-	force_assert(hctx);
-
+	handler_ctx *hctx = ck_calloc(1, sizeof(*hctx));
 	hctx->response = chunk_buffer_acquire();
 	hctx->fd = -1;
 	hctx->fdtocgi = -1;
-
 	return hctx;
 }
 
@@ -116,13 +112,9 @@ static void cgi_handler_ctx_free(handler_ctx *hctx) {
 }
 
 INIT_FUNC(mod_cgi_init) {
-	plugin_data *p;
+	plugin_data * const p = ck_calloc(1, sizeof(*p));
 	const char *s;
 
-	p = calloc(1, sizeof(*p));
-
-	force_assert(p);
-
 	/* for valgrind */
 	s = getenv("LD_PRELOAD");
 	if (s) buffer_copy_string((p->env.ld_preload = buffer_init()), s);
@@ -271,8 +263,7 @@ static int mod_cgi_str_to_signal (const char *s, int default_sig) {
 }
 
 static cgi_limits * mod_cgi_parse_limits(const array * const a, log_error_st * const errh) {
-    cgi_limits * const limits = calloc(1, sizeof(cgi_limits));
-    force_assert(limits);
+    cgi_limits * const limits = ck_calloc(1, sizeof(cgi_limits));
     for (uint32_t i = 0; i < a->used; ++i) {
         const data_unset * const du = a->data[i];
         int32_t v = config_plugin_value_to_int32(du, -1);
@@ -385,8 +376,7 @@ SETDEFAULTS_FUNC(mod_cgi_set_defaults) {
 
 
 static cgi_pid_t * cgi_pid_add(plugin_data *p, pid_t pid, handler_ctx *hctx) {
-    cgi_pid_t *cgi_pid = malloc(sizeof(cgi_pid_t));
-    force_assert(cgi_pid);
+    cgi_pid_t *cgi_pid = ck_malloc(sizeof(cgi_pid_t));
     cgi_pid->pid = pid;
     cgi_pid->signal_sent = 0;
     cgi_pid->hctx = hctx;
@@ -419,8 +409,8 @@ static void cgi_connection_close_fdtocgi(handler_ctx *hctx) {
 	if (-1 == hctx->fdtocgi) return;
 	struct fdevents * const ev = hctx->ev;
 	fdevent_fdnode_event_del(ev, hctx->fdntocgi);
-	/*fdevent_unregister(ev, hctx->fdtocgi);*//*(handled below)*/
-	fdevent_sched_close(ev, hctx->fdtocgi, 0);
+	/*fdevent_unregister(ev, hctx->fdntocgi);*//*(handled below)*/
+	fdevent_sched_close(ev, hctx->fdntocgi);
 	hctx->fdntocgi = NULL;
 	hctx->fdtocgi = -1;
 }
@@ -436,8 +426,8 @@ static void cgi_connection_close(handler_ctx *hctx) {
 		struct fdevents * const ev = hctx->ev;
 		/* close connection to the cgi-script */
 		fdevent_fdnode_event_del(ev, hctx->fdn);
-		/*fdevent_unregister(ev, hctx->fd);*//*(handled below)*/
-		fdevent_sched_close(ev, hctx->fd, 0);
+		/*fdevent_unregister(ev, hctx->fdn);*//*(handled below)*/
+		fdevent_sched_close(ev, hctx->fdn);
 		hctx->fdn = NULL;
 	}
 
@@ -775,7 +765,6 @@ static int cgi_create_env(request_st * const r, plugin_data * const p, handler_c
 	}
 
 	to_cgi_fds[0] = -1;
-  #ifndef __CYGWIN__
 	if (0 == r->reqbody_length) {
 		/* future: might keep fd open in p->devnull for reuse
 		 * and dup() here, or do not close() (later in this func) */
@@ -807,7 +796,6 @@ static int cgi_create_env(request_st * const r, plugin_data * const p, handler_c
 			to_cgi_fds[1] = -1;
 		}
 	}
-  #endif
 
 	unsigned int bufsz_hint = 16384;
   #ifdef _WIN32
@@ -874,7 +862,7 @@ static int cgi_create_env(request_st * const r, plugin_data * const p, handler_c
 			if (!http_header_request_get(r, HTTP_HEADER_OTHER,
 			                             CONST_STR_LEN("Sec-WebSocket-Key")))
 				cgi_env_add(env, CONST_STR_LEN("HTTP_SEC_WEBSOCKET_KEY"),
-				                 CONST_STR_LEN("MDAwMDAwMDAwMDAwMDAwMAo="));
+				                 CONST_STR_LEN("MDAwMDAwMDAwMDAwMDAwMA=="));
 			/*(Upgrade and Connection should not exist for HTTP/2 request)*/
 			cgi_env_add(env, CONST_STR_LEN("HTTP_UPGRADE"), CONST_STR_LEN("websocket"));
 			cgi_env_add(env, CONST_STR_LEN("HTTP_CONNECTION"), CONST_STR_LEN("upgrade"));
@@ -1254,6 +1242,7 @@ static handler_t cgi_waitpid_cb(server *srv, void *p_d, pid_t pid, int status) {
 }
 
 
+__attribute_cold__
 int mod_cgi_plugin_init(plugin *p);
 int mod_cgi_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
