@@ -97,8 +97,7 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
         for (; -1 != cpv->k_id; ++cpv) {
             switch (cpv->k_id) {
               case 0:{/* fastcgi.server */
-                gw_plugin_config *gw = calloc(1, sizeof(gw_plugin_config));
-                force_assert(gw);
+                gw_plugin_config *gw = ck_calloc(1, sizeof(gw_plugin_config));
                 if (!gw_set_defaults_backend(srv, p, cpv->v.a, gw, 0,
                                              cpk[cpv->k_id].k)) {
                     gw_plugin_config_free(gw);
@@ -515,6 +514,7 @@ static handler_t fcgi_check_extension_2(request_st * const r, void *p_d) {
 }
 
 
+__attribute_cold__
 int mod_fastcgi_plugin_init(plugin *p);
 int mod_fastcgi_plugin_init(plugin *p) {
 	p->version      = LIGHTTPD_VERSION_ID;
