@@ -26,7 +26,7 @@ typedef struct {
 } plugin_data;
 
 INIT_FUNC(mod_rewrite_init) {
-    return calloc(1, sizeof(plugin_data));
+    return ck_calloc(1, sizeof(plugin_data));
 }
 
 FREE_FUNC(mod_rewrite_free) {
@@ -351,6 +351,8 @@ URIHANDLER_FUNC(mod_rewrite_uri_handler) {
     return process_rewrite_rules(r, p, p->conf.rewrite);
 }
 
+
+__attribute_cold__
 int mod_rewrite_plugin_init(plugin *p);
 int mod_rewrite_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
