@@ -26,7 +26,7 @@ typedef struct {
 } plugin_data;
 
 INIT_FUNC(mod_simple_vhost_init) {
-    return calloc(1, sizeof(plugin_data));
+    return ck_calloc(1, sizeof(plugin_data));
 }
 
 FREE_FUNC(mod_simple_vhost_free) {
@@ -196,6 +196,7 @@ static handler_t mod_simple_vhost_docroot(request_st * const r, void *p_data) {
 }
 
 
+__attribute_cold__
 int mod_simple_vhost_plugin_init(plugin *p);
 int mod_simple_vhost_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
