@@ -6,13 +6,13 @@
  */
 #include "first.h"
 
-#include <dbi/dbi.h>
-
 #include <string.h>
 #include <stdlib.h>
 
+#include <dbi/dbi.h>
+
+#include "mod_vhostdb_api.h"
 #include "base.h"
-#include "http_vhostdb.h"
 #include "fdevent.h"
 #include "log.h"
 #include "plugin.h"
@@ -24,7 +24,7 @@
  *   vhostdb.dbi = ( "sql"    => "SELECT docroot FROM vhosts WHERE host='?'"
  *                   "dbtype" => "sqlite3",
  *                   "dbname" => "mydb.sqlite",
- *                   "sqlite_dbdir" => "/path/to/sqlite/dbs/" )
+ *                   "sqlite3_dbdir" => "/path/to/sqlite/dbs/" )
  */
 
 typedef struct {
@@ -104,8 +104,7 @@ static int mod_vhostdb_dbconf_setup (server *srv, const array *opts, void **vdat
      * - encoding, default: database default
      */
 
-    if (!buffer_string_is_empty(sqlquery)
-        && !buffer_is_empty(dbname) && !buffer_is_empty(dbtype)) {
+    if (sqlquery && !buffer_is_blank(sqlquery) && dbname && dbtype) {
         /* create/initialise database */
         vhostdb_config *dbconf;
         dbi_inst dbinst = NULL;
@@ -129,7 +128,7 @@ static int mod_vhostdb_dbconf_setup (server *srv, const array *opts, void **vdat
         for (size_t j = 0; j < opts->used; ++j) {
             data_unset *du = opts->data[j];
             const buffer *opt = &du->key;
-            if (!buffer_string_is_empty(opt)) {
+            if (!buffer_is_blank(opt)) {
                 if (du->type == TYPE_INTEGER) {
                     data_integer *di = (data_integer *)du;
                     dbi_conn_set_option_numeric(dbconn, opt->ptr, di->value);
@@ -142,7 +141,7 @@ static int mod_vhostdb_dbconf_setup (server *srv, const array *opts, void **vdat
             }
         }
 
-        dbconf = (vhostdb_config *)calloc(1, sizeof(*dbconf));
+        dbconf = (vhostdb_config *)ck_calloc(1, sizeof(*dbconf));
         dbconf->dbinst = dbinst;
         dbconf->dbconn = dbconn;
         dbconf->sqlquery = sqlquery;
@@ -186,13 +185,13 @@ static int mod_vhostdb_dbi_query(request_st * const r, void *p_d, buffer *docroo
         if (NULL != (d = strchr(b, '?'))) {
             /* escape the uri.authority */
             char *esc = NULL;
-            size_t len = dbi_conn_escape_string_copy(dbconf->dbconn, r->uri.authority.ptr, &esc);
-            buffer_append_string_len(sqlquery, b, (size_t)(d - b));
-            buffer_append_string_len(sqlquery, esc, len);
+            size_t len = dbi_conn_escape_string_copy(dbconf->dbconn,
+                                                     r->uri.authority.ptr,&esc);
+            buffer_append_str2(sqlquery, b, (size_t)(d - b), esc, len);
             free(esc);
             if (0 == len) return -1;
         } else {
-            d = dbconf->sqlquery->ptr + buffer_string_length(dbconf->sqlquery);
+            d = dbconf->sqlquery->ptr + buffer_clen(dbconf->sqlquery);
             buffer_append_string_len(sqlquery, b, (size_t)(d - b));
             break;
         }
@@ -229,7 +228,7 @@ static int mod_vhostdb_dbi_query(request_st * const r, void *p_d, buffer *docroo
 INIT_FUNC(mod_vhostdb_init) {
     static http_vhostdb_backend_t http_vhostdb_backend_dbi =
       { "dbi", mod_vhostdb_dbi_query, NULL };
-    plugin_data *p = calloc(1, sizeof(*p));
+    plugin_data *p = ck_calloc(1, sizeof(*p));
 
     /* register http_vhostdb_backend_dbi */
     http_vhostdb_backend_dbi.p_d = p;
@@ -328,6 +327,7 @@ SETDEFAULTS_FUNC(mod_vhostdb_set_defaults) {
 }
 
 
+__attribute_cold__
 int mod_vhostdb_dbi_plugin_init (plugin *p);
 int mod_vhostdb_dbi_plugin_init (plugin *p)
 {
