@@ -82,13 +82,23 @@
 #include <sys/ioctl.h>
 #endif
 #if defined(__APPLE__) && defined(__MACH__)
-#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 101000 /* OS X 10.10+ */
+#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 101200 /* OS X 10.12+ */
 #undef HAVE_ARC4RANDOM_BUF
 #define HAVE_CCRANDOMGENERATEBYTES
+#include <CommonCrypto/CommonCryptoError.h>
 #include <CommonCrypto/CommonRandom.h>
 #endif
 #endif
 
+#ifdef __has_include
+#if __has_include(<sys/auxv.h>)
+#include <sys/auxv.h> /* getauxval(AT_RANDOM) is a glibc extension */
+#ifdef AT_RANDOM
+#define HAVE_GETAUXVAL
+#endif
+#endif
+#endif
+
 /* Take some reasonable steps to attempt to *seed* random number generators with
  * cryptographically random data.  Some of these initialization routines may
  * block, and are intended to be called only at startup in lighttpd, or
@@ -97,7 +107,7 @@
  * Update: li_rand_init() is now deferred until first use so that installations
  * that do not use modules which use these routines do need to potentially block
  * at startup.  Current use by core lighttpd modules is in mod_auth HTTP Digest
- * auth and in mod_usertrack.  Deferring collection of random data until first
+ * auth.  Deferring collection of random data until first
  * use may allow sufficient entropy to be collected by kernel before first use,
  * helping reduce or avoid situations in low-entropy-generating embedded devices
  * which might otherwise block lighttpd for minutes at device startup.
@@ -261,10 +271,24 @@ static void li_rand_init (void)
         u = (unsigned int)(time(NULL) ^ getpid());
       #else
         /* NOTE: not cryptographically random !!! */
+       #ifdef HAVE_GETAUXVAL
+        char *auxv_random = (char *)(uintptr_t)getauxval(AT_RANDOM);
+        if (auxv_random) {
+            memcpy(&u, auxv_random, 4);
+            memcpy(xsubi, auxv_random+4, 6);
+        }
+        else
+            memset(xsubi, (u = 0), sizeof(xsubi));
+        srand((unsigned int)(time(NULL) ^ getpid()) ^ u);
+        for (u = 0; u < sizeof(unsigned short); ++u)
+            /* coverity[dont_call : FALSE] */
+            xsubi[u] ^= (unsigned short)(rand() & 0xFFFF);
+       #else
         srand((unsigned int)(time(NULL) ^ getpid()));
         for (u = 0; u < sizeof(unsigned short); ++u)
             /* coverity[dont_call : FALSE] */
             xsubi[u] = (unsigned short)(rand() & 0xFFFF);
+       #endif
         u = ((unsigned int)xsubi[0] << 16) | xsubi[1];
       #endif
     }
@@ -386,9 +410,9 @@ int li_rand_pseudo (void)
         return i;
   #endif
   #ifdef HAVE_CCRANDOMGENERATEBYTES
-    int i;
-    if (CCRandomGenerateBytes(&i, sizeof(i)) == kCCSuccess)
-        return i;
+    int j;
+    if (CCRandomGenerateBytes(&j, sizeof(j)) == kCCSuccess)
+        return j;
   #endif
   #ifdef HAVE_ARC4RANDOM_BUF
     return (int)arc4random();
