@@ -40,4 +40,24 @@
 #undef USE_LIB_CRYPTO
 #endif
 
+#ifdef USE_MBEDTLS_CRYPTO
+#ifdef FORCE_MBEDTLS_CRYPTO
+#undef USE_GNUTLS_CRYPTO
+#undef USE_NETTLE_CRYPTO
+#undef USE_NSS_CRYPTO
+#undef USE_OPENSSL_CRYPTO
+#undef USE_WOLFSSL_CRYPTO
+#endif
+#endif
+
+#ifdef USE_WOLFSSL_CRYPTO
+#ifdef FORCE_WOLFSSL_CRYPTO
+#undef USE_GNUTLS_CRYPTO
+#undef USE_MBEDTLS_CRYPTO
+#undef USE_NETTLE_CRYPTO
+#undef USE_NSS_CRYPTO
+#undef USE_OPENSSL_CRYPTO
+#endif
+#endif
+
 #endif
